import dataclasses
from typing import List, Optional

from jig.collector.domain.source_code.source_code import SourceCode


@dataclasses.dataclass(frozen=True)
class SourceCodeCollection:
    collection: List[SourceCode]

    def __iter__(self):
        return self.collection.__iter__()

    def __len__(self):
        return self.collection.__len__()

    def __getitem__(self, item):
        return self.collection.__getitem__(item)

    def get_by_relative_path(self, relative_path: str) -> Optional[SourceCode]:
        for source_code in self.collection:
            source_file_path = source_code.file.source_file_path
            if str(source_file_path.relative_path_from_root) == relative_path:
                return source_code
        return None



import dataclasses

from graphviz import Digraph

from jig.visualizer.module_dependency.domain.model.graph import Graph
from .cluster_renderer import ClusterRenderer


@dataclasses.dataclass
class GraphRenderer:
    graph: Graph

    def render(self) -> Digraph:
        d = Digraph()

        for node in sorted(self.graph.nodes):
            node_options = node.style.to_dict()
            # 描画する node の配下に位置する node が存在する場合には、 shape=rect として表示する
            if self.graph.child_node_exists(node):
                node_options["shape"] = "rect"
            d.node(name=node.path.name, **node_options)

        for edge in sorted(self.graph.edges):
            d.edge(edge.tail.name, edge.head.name, **edge.style.to_dict())

        for cluster in self.graph.clusters.values():
            cluster_renderer = ClusterRenderer(cluster)
            d.subgraph(cluster_renderer.render())

        return d



import os
from pathlib import Path
from typing import Optional, List

import fire

from jig.analyzer.domain.dependency.import_dependency import ImportDependencyCollection
from jig.collector.application import SourceCodeCollector
from jig.collector.domain.source_code.source_code import SourceCode
from jig.collector.domain.source_code.source_code_collection import SourceCodeCollection


class Main:
    def modules(self, target_path: str, root_path: Optional[str] = None) -> None:
        """
        指定されたパスのモジュール名を表示します。
        ディレクトリを指定した場合は再帰的にPythonファイルを収集し、全てのソースコード
        のモジュール名をリストで表示します。
        :param target_path: 解析対象のソースファイルまたはディレクトリへのパス
        :param root_path: 解析対象Pythonコードのルートディレクトリパス
        :return:
        """
        source_codes = self._collect_source_codes(
            target_path=target_path, root_path=root_path
        )
        for source_code in source_codes:
            print(str(source_code.module_path))

    def module_imports(self, target_path: str, root_path: Optional[str] = None) -> None:
        """
        指定されたパスのモジュールのインポートしているモジュールリストを表示します。
        ディレクトリを指定した場合は再帰的にPythonファイルを収集し、全てのソースコード
        のモジュールのインポートしているモジュールをリストで表示します。
        :param target_path: 解析対象のソースファイルまたはディレクトリへのパス
        :param root_path: 解析対象Pythonコードのルートディレクトリパス
        :return:
        """
        source_codes = self._collect_source_codes(
            target_path=target_path, root_path=root_path
        )
        for source_code in source_codes:
            for import_module in source_code.import_paths:
                print(str(source_code.module_path), str(import_module.module_path))

    def module_deps(self, target_path: str, root_path: Optional[str] = None) -> None:
        source_codes = SourceCodeCollection(
            self._collect_source_codes(target_path=target_path, root_path=root_path)
        )

        collection = ImportDependencyCollection.build_from_source_code_collection(
            source_codes
        )
        dependencies = collection.build_module_dependencies()

        for dependency in dependencies:
            print(str(dependency.src), str(dependency.dest))

    def source_codes(self, target_path: str, root_path: Optional[str] = None) -> None:
        """
        指定されたパスのPythonソースコード解析結果を表示します。
        ディレクトリを指定した場合は再帰的にPythonファイルを収集し、全てのソースコード
        解析結果を表示します。
        :param target_path: 解析対象のソースファイルまたはディレクトリへのパス
        :param root_path: 解析対象Pythonコードのルートディレクトリパス
        :return:
        """
        source_codes = self._collect_source_codes(
            target_path=target_path, root_path=root_path
        )
        print(source_codes)

    def _collect_source_codes(
        self, target_path: str, root_path: Optional[str]
    ) -> List[SourceCode]:
        if not root_path:
            root_path = os.getcwd()

        return list(
            SourceCodeCollector(root_path=Path(root_path)).collect(
                target_path=Path(target_path)
            )
        )


def main():
    fire.Fire(Main)


if __name__ == "__main__":
    main()



import dataclasses
from typing import Optional, List, Tuple, Iterator

from jig.visualizer.module_dependency.domain.value.module_edge import (
    ModuleEdge,
    ModuleEdgeCollection,
)
from jig.visualizer.module_dependency.domain.value.module_node import ModuleNode


@dataclasses.dataclass
class MasterGraph:
    edges: ModuleEdgeCollection = dataclasses.field(
        default_factory=ModuleEdgeCollection
    )

    @classmethod
    def from_tuple_list(cls, edges: List[Tuple[str, str]]) -> "MasterGraph":
        return cls(ModuleEdgeCollection.from_tuple_list(edges))

    def find_parent_edge(self, edge: ModuleEdge) -> Optional[ModuleEdge]:
        return self.edges.find_parent_edge(edge=edge)

    def find_nodes(self, node: ModuleNode) -> List[ModuleNode]:
        result = []

        for new_edge in self.edges:
            if new_edge.head.belongs_to(node):
                result.append(new_edge.head)
            if new_edge.tail.belongs_to(node):
                result.append(new_edge.tail)

        return result

    def find_edges(self, node: ModuleNode) -> List[ModuleEdge]:
        result = []

        for new_edge in self.edges:
            if new_edge.head.belongs_to(node) and new_edge.tail.belongs_to(node):
                result.append(new_edge)

        return result

    def child_node_exists(self, node: ModuleNode) -> bool:
        for edge in self.edges:
            for other_node in [edge.head, edge.tail]:
                # node よりも path_level の大きい (深い階層) の node が存在するか？ をチェックする
                if other_node.path_level > node.path_level and other_node.belongs_to(
                    node
                ):
                    return True
        return False

    def __iter__(self) -> Iterator[ModuleEdge]:
        return iter(self.edges)

    def to_dict(self) -> dict:
        edges = sorted([(e.tail.name, e.head.name) for e in self.edges])
        return {"edges": edges}



import dataclasses
from typing import List

from jig.collector.domain.ast import ClassDef, JigSourceCode
from jig.collector.domain.source_code.source_code_import_dependency import (
    SourceCodeImportDependency,
)
from jig.collector.domain.values.import_path_collection import ImportPathCollection
from jig.collector.domain.values.module_path import ModulePath
from jig.collector.domain.source_file.source_file import SourceFile


@dataclasses.dataclass(frozen=True)
class SourceCode:
    file: SourceFile
    import_paths: ImportPathCollection
    class_defs: List[ClassDef]

    @property
    def module_path(self) -> ModulePath:
        return self.file.module_path

    @classmethod
    def build(cls, file: SourceFile) -> "SourceCode":
        jig_source_code = JigSourceCode.build(
            source=file.content, filename=file.filename
        )

        return SourceCode(
            file=file,
            import_paths=cls._build_import_paths(file, jig_source_code),
            class_defs=jig_source_code.class_defs,
        )

    @classmethod
    def _build_import_paths(
        cls, file: SourceFile, jig_source_code: JigSourceCode
    ) -> ImportPathCollection:
        import_paths = ImportPathCollection()

        for import_ast in jig_source_code.imports:
            import_paths += ImportPathCollection.build_by_import_ast(import_ast)

        for import_from_ast in jig_source_code.import_froms:
            import_paths += ImportPathCollection.build_by_import_from_ast(
                file_path=file.source_file_path, import_from=import_from_ast
            )

        return import_paths

    def build_import_dependency(self) -> SourceCodeImportDependency:
        return SourceCodeImportDependency.build(
            source_module_path=self.file.module_path, import_paths=self.import_paths
        )



import dataclasses

from jig.collector.domain.values.module_path import ModulePath
from jig.collector.domain.source_file.source_file_path import SourceFilePath


@dataclasses.dataclass(frozen=True)
class SourceFile:
    source_file_path: SourceFilePath
    size: int
    content: str = dataclasses.field(repr=False)

    @property
    def filename(self):
        return self.source_file_path.filename

    @property
    def module_path(self) -> ModulePath:
        return self.source_file_path.module_path



import os
import subprocess
import textwrap
from typing import List

from jig.analyzer.domain.dependency.module_dependency import ModuleDependency


class ModuleDependencyVisualizer:
    def __init__(self, dependencies: List[ModuleDependency]):
        self.dependencies = dependencies

    def dot_text(self, depth: int) -> str:
        result = []
        for dep in self.dependencies:
            path1 = str(dep.src.path_in_depth(depth))
            path2 = str(dep.dest.path_in_depth(depth))

            # 自己参照は依存関係分析的には意味ないので除く
            if path1 == path2:
                continue

            result.append(f'"{path1}" -> "{path2}";')

        # 設定テキスト
        setting_text = """
          graph [
            rankdir = "LR",
            overlap = "scale",
            ratio = "fill",
            fontsize = "24",
            fontname = "Helvetica",
            clusterrank = "local"
            dpi = 180
          ]

          node [
            fontsize=7
            shape=ellipse
          ];
        """

        # 重複削除とソート
        edges = sorted(list(set(result)))
        edge_text = "\n".join(edges)

        graph_text = [
            "digraph {",
            textwrap.indent(textwrap.dedent(setting_text), "  "),
            textwrap.indent(edge_text, "  "),
            "}",
        ]

        return "\n".join(graph_text)

    def render_dot_text(self, depth: int, output_dir: str) -> None:
        os.makedirs(output_dir, exist_ok=True)

        filepath = os.path.join(output_dir, f"dependency{depth}.dot")

        with open(filepath, mode="w") as f:
            f.write(self.dot_text(depth))

    def visualize(self, depth: int, output_dir: str) -> None:
        dot = self.dot_text(depth)

        os.makedirs(output_dir, exist_ok=True)

        filepath = os.path.join(output_dir, f"dependency{depth}.png")

        p = subprocess.Popen(["dot", "-Tpng", f"-o{filepath}"], stdin=subprocess.PIPE)

        p.communicate(dot.encode())
